@@ -109,7 +109,7 @@ static int acurite_rain_896_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     // This needs more validation to positively identify correct sensor type, but it basically works if message is really from acurite raingauge and it doesn't have any errors
     if (bitbuffer->bits_per_row[0] < 24)
         return DECODE_ABORT_LENGTH;
-    
+
     if ((b[0] == 0) || (b[1] == 0) || (b[2] == 0) || (b[3] != 0) || (b[4] != 0))
         return DECODE_ABORT_EARLY;
 
@@ -159,7 +159,7 @@ static int acurite_th_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 
     for (uint16_t brow = 0; brow < bitbuffer->num_rows; ++brow) {
         if (bitbuffer->bits_per_row[brow] != 40) {
-           continue;
+           continue; // DECODE_ABORT_LENGTH
         }
 
         bb = bitbuffer->bb[brow];
@@ -167,7 +167,7 @@ static int acurite_th_decode(r_device *decoder, bitbuffer_t *bitbuffer)
         cksum = (bb[0] + bb[1] + bb[2] + bb[3]);
 
         if (cksum == 0 || ((cksum & 0xff) != bb[4])) {
-            continue;
+            continue; // DECODE_FAIL_MIC
         }
 
         // Temperature in Celsius is encoded as a 12 bit integer value
@@ -185,9 +185,10 @@ static int acurite_th_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 "model",            "",             DATA_STRING, _X("Acurite-609TXC","Acurite 609TXC Sensor"),
                 "id",               "",             DATA_INT,    id,
                 "battery",          "",             DATA_STRING, battery_low ? "LOW" : "OK",
-                "status",           "",             DATA_INT,    status,
                 "temperature_C",    "Temperature",  DATA_FORMAT, "%.1f C", DATA_DOUBLE, tempc,
                 "humidity",         "Humidity",     DATA_INT,    humidity,
+                "status",           "",             DATA_INT,    status,
+                "mic",              "Integrity",    DATA_STRING, "CHECKSUM",
                 NULL);
         /* clang-format on */
 
@@ -311,9 +312,8 @@ Additional reverse engineering needed:
 @todo - figure out remaining status bits and how to report
 */
 
-static int acurite_6045_decode(r_device *decoder, bitrow_t bb, int browlen)
+static int acurite_6045_decode(r_device *decoder, bitbuffer_t *bitbuffer, unsigned row)
 {
-    int valid = 0;
     float tempf;
     uint8_t humidity, message_type, l_status;
     char channel, channel_str[2];
@@ -324,6 +324,9 @@ static int acurite_6045_decode(r_device *decoder, bitrow_t bb, int browlen)
     int exception = 0;
     data_t *data;
 
+    int browlen = (bitbuffer->bits_per_row[row] + 7) / 8;
+    uint8_t *bb = bitbuffer->bb[row];
+
     channel = acurite_getChannel(bb[0]);  // same as TXR
     sprintf(channel_str, "%c", channel);  // No DATA_CHAR, need null term. str
 
@@ -368,7 +371,7 @@ static int acurite_6045_decode(r_device *decoder, bitrow_t bb, int browlen)
         exception++;
     }
 
-    // FIXME - temporarily leaving the old output for ease of debugging
+    // TODO: temporarily leaving the old output for ease of debugging
     // and backward compatibility. Remove when doing a "1.0" release.
     if (decoder->verbose) {
         fprintf(stderr, "Acurite lightning 0x%04X Ch %c Msg Type 0x%02x: %.1f F %d %% RH Strikes %d Distance %d L_status 0x%02x -",
@@ -386,6 +389,7 @@ static int acurite_6045_decode(r_device *decoder, bitrow_t bb, int browlen)
             "model",            "",                 DATA_STRING, _X("Acurite-6045M","Acurite Lightning 6045M"),
             "id",               NULL,               DATA_INT,    sensor_id,
             "channel",          NULL,               DATA_STRING, channel_str,
+            "battery",          "battery",          DATA_STRING, battery_low ? "LOW" : "OK",
             "temperature_F",    "temperature",      DATA_FORMAT, "%.1f F",     DATA_DOUBLE,     tempf,
             "humidity",         "humidity",         DATA_INT,    humidity,
             "strike_count",     "strike_count",     DATA_INT,    strike_count,
@@ -393,16 +397,13 @@ static int acurite_6045_decode(r_device *decoder, bitrow_t bb, int browlen)
             "active",           "active_mode",      DATA_INT,    active,    // @todo convert to bool
             "rfi",              "rfi_detect",       DATA_INT,    rfi_detect,     // @todo convert to bool
             "ussb1",            "unk_status1",      DATA_INT,    ussb1,    // @todo convert to bool
-            "battery",          "battery",          DATA_STRING, battery_low ? "LOW" : "OK",
             "exception",        "data_exception",   DATA_INT,    exception,    // @todo convert to bool
             "raw_msg",          "raw_message",      DATA_STRING, raw_str,
             NULL);
     /* clang-format on */
 
     decoder_output_data(decoder, data);
-    valid++;
-
-    return valid;
+    return 1;
 }
 
 /**
@@ -450,7 +451,7 @@ static int acurite_txr_decode(r_device *decoder, bitbuffer_t *bitbuffer)
             bitbuffer->bits_per_row[brow] != ACURITE_6045_BITLEN) {
             if (decoder->verbose > 1 && bitbuffer->bits_per_row[brow] > 16)
                 fprintf(stderr, "%s: skipping wrong len\n", __func__);
-            continue;
+            continue; // DECODE_ABORT_LENGTH
         }
 
         // There will be 1 extra false zero bit added by the demod.
@@ -464,12 +465,12 @@ static int acurite_txr_decode(r_device *decoder, bitbuffer_t *bitbuffer)
         if (sum == 0 || (sum & 0xff) != bb[browlen - 1]) {
             if (decoder->verbose)
                 bitrow_printf(bb, bitbuffer->bits_per_row[brow], "%s: bad checksum: ", __func__);
-            continue;
+            continue; // DECODE_FAIL_MIC
         }
 
         if (decoder->verbose) {
             fprintf(stderr, "%s: Parity: ", __func__);
-            for (uint8_t i = 0; i < browlen; i++) {
+            for (int i = 0; i < browlen; i++) {
                 fprintf(stderr, "%d", parity8(bb[i]));
             }
             fprintf(stderr,"\n");
@@ -507,11 +508,11 @@ static int acurite_txr_decode(r_device *decoder, bitbuffer_t *bitbuffer)
             data = data_make(
                     "model",                "",             DATA_STRING, _X("Acurite-Tower","Acurite tower sensor"),
                     "id",                   "",             DATA_INT,    sensor_id,
-                    "sensor_id",            NULL,           DATA_FORMAT, "0x%04x",   DATA_INT, sensor_id, // TODO: hex output not working, delete at 1.0 release
                     "channel",              NULL,           DATA_STRING, &channel_str,
+                    _X("battery_ok","battery_low"), "",     DATA_INT,    _X(!battery_low,battery_low),
                     "temperature_C",        "Temperature",  DATA_FORMAT, "%.1f C", DATA_DOUBLE, tempc,
                     "humidity",             "Humidity",     DATA_INT,    humidity,
-                    _X("battery_ok","battery_low"), "",     DATA_INT,    _X(!battery_low,battery_low),
+                    "mic",                  "Integrity",    DATA_STRING, "CHECKSUM",
                     NULL);
             /* clang-format on */
 
@@ -562,14 +563,15 @@ static int acurite_txr_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 /* clang-format off */
                 data = data_make(
                         "model",        "",   DATA_STRING,    _X("Acurite-5n1","Acurite 5n1 sensor"),
+                        _X("subtype","message_type"), NULL,   DATA_INT,       message_type,
                         _X("id", "sensor_id"),    NULL, DATA_INT,       sensor_id,
                         "channel",      NULL,   DATA_STRING,    &channel_str,
                         "sequence_num",  NULL,   DATA_INT,      sequence_num,
                         "battery",      NULL,   DATA_STRING,    battery_low ? "OK" : "LOW",
-                        "message_type", NULL,   DATA_INT,       message_type,
                         _X("wind_avg_km_h","wind_speed_kph"),   "wind_speed",   DATA_FORMAT,    "%.1f km/h", DATA_DOUBLE,     wind_speed_kph,
                         "wind_dir_deg", NULL,   DATA_FORMAT,    "%.1f", DATA_DOUBLE,    wind_dir,
                         _X("rain_in","rain_inch"), "Rainfall Accumulation",   DATA_FORMAT, "%.2f in", DATA_DOUBLE, raincounter * 0.01f,
+                        "mic",                  "Integrity",    DATA_STRING, "CHECKSUM",
                         NULL);
                 /* clang-format on */
 
@@ -588,14 +590,15 @@ static int acurite_txr_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 /* clang-format off */
                 data = data_make(
                         "model",        "",   DATA_STRING,    _X("Acurite-5n1","Acurite 5n1 sensor"),
+                        _X("subtype","message_type"), NULL,   DATA_INT,       message_type,
                         _X("id", "sensor_id"),    NULL, DATA_INT,  sensor_id,
                         "channel",      NULL,   DATA_STRING,    &channel_str,
                         "sequence_num",  NULL,   DATA_INT,      sequence_num,
                         "battery",      NULL,   DATA_STRING,    battery_low ? "OK" : "LOW",
-                        "message_type", NULL,   DATA_INT,       message_type,
                         _X("wind_avg_km_h","wind_speed_kph"),   "wind_speed",   DATA_FORMAT,    "%.1f km/h", DATA_DOUBLE,     wind_speed_kph,
                         "temperature_F",     "temperature",    DATA_FORMAT,    "%.1f F", DATA_DOUBLE,    tempf,
                         "humidity",     NULL,    DATA_FORMAT,    "%d",   DATA_INT,   humidity,
+                        "mic",                  "Integrity",    DATA_STRING, "CHECKSUM",
                         NULL);
                 /* clang-format on */
 
@@ -616,14 +619,15 @@ static int acurite_txr_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 /* clang-format off */
                 data = data_make(
                         "model",        "",   DATA_STRING,    _X("Acurite-3n1","Acurite 3n1 sensor"),
+                        _X("subtype","message_type"), NULL,   DATA_INT,       message_type,
                         _X("id", "sensor_id"),    NULL,   DATA_FORMAT,    "0x%02X",   DATA_INT,       sensor_id,
                         "channel",      NULL,   DATA_STRING,    &channel_str,
                         "sequence_num",  NULL,   DATA_INT,      sequence_num,
                         "battery",      NULL,   DATA_STRING,    battery_low ? "OK" : "LOW",
-                        "message_type", NULL,   DATA_INT,       message_type,
                         _X("wind_avg_mi_h","wind_speed_mph"),   "wind_speed",   DATA_FORMAT,    "%.1f mi/h", DATA_DOUBLE,     wind_speed_mph,
                         "temperature_F",     "temperature",    DATA_FORMAT,    "%.1f F", DATA_DOUBLE,    tempf,
                         "humidity",     NULL,    DATA_FORMAT,    "%d",   DATA_INT,   humidity,
+                        "mic",                  "Integrity",    DATA_STRING, "CHECKSUM",
                         NULL);
                 /* clang-format on */
 
@@ -634,7 +638,7 @@ static int acurite_txr_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 // Rain Fall Gauge 899
                 // The high 2 bits of byte zero are the channel (bits 7,6), 00 = A, 01 = B, 10 = C
                 channel     = bb[0] >> 6;
-                raincounter = ((bb[5] & 0x7f) << 7) | (bb[6] & 0x7f); // TODO: a tip is 1mm?
+                raincounter = ((bb[5] & 0x7f) << 7) | (bb[6] & 0x7f); // one tip is 0.01 inch, i.e. 0.254mm
 
                 /* clang-format off */
                 data = data_make(
@@ -642,7 +646,8 @@ static int acurite_txr_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                         "id",               "",                         DATA_INT,    sensor_id,
                         "channel",          "",                         DATA_INT,    channel,
                         "battery_ok",       "Battery",                  DATA_INT,    !battery_low,
-                        "rain_mm",          "Rainfall Accumulation",    DATA_FORMAT, "%d mm", DATA_DOUBLE, raincounter,
+                        "rain_mm",          "Rainfall Accumulation",    DATA_FORMAT, "%.2f mm", DATA_DOUBLE, raincounter * 0.254,
+                        "mic",                  "Integrity",    DATA_STRING, "CHECKSUM",
                         NULL);
                 /* clang-format on */
 
@@ -650,14 +655,16 @@ static int acurite_txr_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 valid++;
             }
             else {
+                if (decoder->verbose > 1) {
                 fprintf(stderr, "%s: Acurite 5n1 sensor 0x%04X Ch %c, Status %02X, Unknown message type 0x%02x\n",
                     __func__, sensor_id, channel, bb[3], message_type);
+                }
             }
         }
 
         else if (browlen == ACURITE_6045_BITLEN / 8) {
             // TODO: check parity and reject if invalid
-            valid += acurite_6045_decode(decoder, bb, browlen);
+            valid += acurite_6045_decode(decoder, bitbuffer, brow);
         }
 
     }
@@ -722,7 +729,7 @@ static int acurite_986_decode(r_device *decoder, bitbuffer_t *bitbuffer)
             bitbuffer->bits_per_row[brow] > 43 ) {
             if (decoder->verbose > 1 && bitbuffer->bits_per_row[brow] > 16)
                 fprintf(stderr,"%s: skipping wrong len\n", __func__);
-            continue;
+            continue; // DECODE_ABORT_LENGTH
         }
         bb = bitbuffer->bb[brow];
 
@@ -730,11 +737,11 @@ static int acurite_986_decode(r_device *decoder, bitbuffer_t *bitbuffer)
         // may eliminate these with a better PPM (precise?) demod.
         if ((bb[0] == 0xff && bb[1] == 0xff && bb[2] == 0xff) ||
                 (bb[0] == 0x00 && bb[1] == 0x00 && bb[2] == 0x00)) {
-            continue;
+            continue; // DECODE_ABORT_EARLY
         }
 
         // Reverse the bits, msg sent LSB first
-        for (uint8_t i = 0; i < browlen; i++)
+        for (int i = 0; i < browlen; i++)
             br[i] = reverse8(bb[i]);
 
         if (decoder->verbose)
@@ -766,7 +773,7 @@ static int acurite_986_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                     fprintf(stderr, "%s: CRC fix %02x - %02x\n", __func__, crc, crcc);
             }
             else {
-                continue;
+                continue; // DECODE_FAIL_MIC
             }
         }
 
@@ -782,9 +789,10 @@ static int acurite_986_decode(r_device *decoder, bitbuffer_t *bitbuffer)
                 "model",            "",             DATA_STRING, _X("Acurite-986","Acurite 986 Sensor"),
                 "id",               NULL,           DATA_INT,    sensor_id,
                 "channel",          NULL,           DATA_STRING, channel_str,
-                "temperature_F",    "temperature",  DATA_FORMAT, "%f F", DATA_DOUBLE,    (float)tempf,
                 "battery",          "battery",      DATA_STRING, battery_low ? "LOW" : "OK",
+                "temperature_F",    "temperature",  DATA_FORMAT, "%f F", DATA_DOUBLE,    (float)tempf,
                 "status",           "status",       DATA_INT,    status,
+                "mic",              "Integrity",    DATA_STRING, "CRC",
                 NULL);
         /* clang-format on */
 
@@ -811,19 +819,19 @@ static int acurite_606_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 
     row = bitbuffer_find_repeated_row(bitbuffer, 3, 32); // expected are 6 rows
     if (row < 0)
-        return 0;
+        return DECODE_ABORT_EARLY;
 
     if (bitbuffer->bits_per_row[row] > 33)
-        return 0;
+        return DECODE_ABORT_LENGTH;
 
     b = bitbuffer->bb[row];
 
     if (b[4] != 0)
-        return 0;
+        return DECODE_FAIL_SANITY;
 
     // reject all blank messages
     if (b[0] == 0 && b[1] == 0 && b[2] == 0 && b[3] == 0)
-        return 0;
+        return DECODE_FAIL_SANITY;
 
     if (decoder->verbose > 1)
         bitbuffer_printf(bitbuffer, "%s: ", __func__);
@@ -831,7 +839,7 @@ static int acurite_606_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     // calculate the checksum and only continue if we have a matching checksum
     uint8_t chk = lfsr_digest8(b, 3, 0x98, 0xf1);
     if (chk != b[3])
-        return 0;
+        return DECODE_FAIL_MIC;
 
     // Processing the temperature:
     // Upper 4 bits are stored in nibble 1, lower 8 bits are stored in nibble 2
@@ -858,7 +866,7 @@ static int acurite_606_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 
 static int acurite_00275rm_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 {
-    int crc, battery_low, id, model, valid = 0;
+    int crc, battery_low, id, model_flag, valid = 0;
     data_t *data;
     float tempc, ptempc;
     uint8_t probe, humidity, phumidity, water;
@@ -873,8 +881,10 @@ static int acurite_00275rm_decode(r_device *decoder, bitbuffer_t *bitbuffer)
 
     //  This sensor repeats signal three times.  Store each copy.
     for (uint16_t brow = 0; brow < bitbuffer->num_rows; ++brow) {
-        if (bitbuffer->bits_per_row[brow] != 88) continue;
-        if (nsignal>=3) continue;
+        if (bitbuffer->bits_per_row[brow] != 88)
+          continue; // DECODE_ABORT_LENGTH
+        if (nsignal>=3)
+          continue; // DECODE_ABORT_EARLY
         memcpy(signal[nsignal], bitbuffer->bb[brow], 11);
         if (decoder->verbose)
             bitrow_printf(signal[nsignal], 11 * 8, "%s: ", __func__);
@@ -900,15 +910,15 @@ static int acurite_00275rm_decode(r_device *decoder, bitbuffer_t *bitbuffer)
             //  Decode the combined signal
             id          = (signal[0][0] << 16) | (signal[0][1] << 8) | signal[0][3];
             battery_low = (signal[0][2] & 0x40) == 0;
-            model       = (signal[0][2] & 1);
+            model_flag  = (signal[0][2] & 1);
             tempc       = ((signal[0][4] << 4) | (signal[0][5] >> 4)) * 0.1 - 100;
             probe       = signal[0][5] & 3;
             humidity    = ((signal[0][6] & 0x1f) << 2) | (signal[0][7] >> 6);
             //  No probe
             /* clang-format off */
             data = data_make(
-                    "model",           "",             DATA_STRING,    model ? _X("Acurite-00275rm","00275rm") : _X("Acurite-00276rm","00276rm"),
-                    "probe",           "",             DATA_INT,       probe,
+                    "model",           "",             DATA_STRING,    model_flag ? _X("Acurite-00275rm","00275rm") : _X("Acurite-00276rm","00276rm"),
+                    _X("subtype","probe"), "Probe",    DATA_INT,       probe,
                     "id",              "",             DATA_INT,       id,
                     "battery",         "",             DATA_STRING,    battery_low ? "LOW" : "OK",
                     "temperature_C",   "Celsius",      DATA_FORMAT,    "%.1f C",  DATA_DOUBLE, tempc,
@@ -989,9 +999,10 @@ static char *acurite_th_output_fields[] = {
     "model",
     "id",
     "battery",
-    "status",
     "temperature_C",
     "humidity",
+    "status",
+    "mic",
     NULL,
 };
 
@@ -1013,16 +1024,18 @@ r_device acurite_th = {
  */
 static char *acurite_txr_output_fields[] = {
     "model",
+    "subtype",
+    "message_type", // TODO: remove this
     "id",
-    "sensor_id",
+    "sensor_id", // TODO: remove this
     "channel",
-    "temperature_C",
-    "humidity",
+    "sequence_num",
     "battery_low", // TODO: remove this
     "battery_ok",
-    "sequence_num",
     "battery",
-    "message_type",
+    "temperature_C",
+    "temperature_F",
+    "humidity",
     "wind_speed_mph", // TODO: remove this
     "wind_speed_kph", // TODO: remove this
     "wind_avg_mi_h",
@@ -1031,7 +1044,6 @@ static char *acurite_txr_output_fields[] = {
     "rain_inch", // TODO: remove this
     "rain_in",
     "rain_mm",
-    "temperature_F",
     NULL,
 };
 
@@ -1062,8 +1074,8 @@ static char *acurite_986_output_fields[] = {
     "model",
     "id",
     "channel",
-    "temperature_F",
     "battery",
+    "temperature_F",
     "status",
     NULL,
 };
@@ -1116,7 +1128,8 @@ r_device acurite_606 = {
 
 static char *acurite_00275rm_output_fields[] = {
     "model",
-    "probe",
+    "subtype",
+    "probe", // TODO: remove this
     "id",
     "battery",
     "temperature_C",
