@@ -106,14 +106,14 @@ static int dish_remote_6_3_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 
     r = bitbuffer_find_repeated_row(bitbuffer, MYDEVICE_MINREPEATS, MYDEVICE_BITLEN);
     if (r < 0 || bitbuffer->bits_per_row[r] > MYDEVICE_BITLEN) {
-        return 0;
+        return DECODE_ABORT_LENGTH;
     }
 
     b = bitbuffer->bb[r];
 
     /* Check fixed bits to prevent misreads */
     if ((b[0] & 0x03) != 0x02 || (b[1] & 0xe8) != 0xa8) {
-        return 0;
+        return DECODE_FAIL_SANITY;
     }
 
     button = b[0] >> 2;
