@@ -52,16 +52,16 @@ static int maverick_et73_sensor_callback(r_device *decoder, bitbuffer_t *bitbuff
     // The device transmits many rows, let's check for 3 matching.
     row = bitbuffer_find_repeated_row(bitbuffer, 3, 48);
     if (row < 0) {
-        return 0;
+        return DECODE_ABORT_EARLY;
     }
 
     bytes = bitbuffer->bb[row];
     if (!bytes[0] && !bytes[1] && !bytes[2] && !bytes[3]) {
-        return 0; // reduce false positives
+        return DECODE_ABORT_EARLY; // reduce false positives
     }
 
     if (bitbuffer->bits_per_row[row] != 48)
-        return 0;
+        return DECODE_ABORT_LENGTH;
 
     device = bytes[0];
 
