@@ -23,8 +23,8 @@ static int rftech_callback(r_device *decoder, bitbuffer_t *bitbuffer) {
 
     r = bitbuffer_find_repeated_row(bitbuffer, 3, 24);
 
-    if(r < 0 || bitbuffer->bits_per_row[r] != 24)
-        return 0;
+    if (r < 0 || bitbuffer->bits_per_row[r] != 24)
+        return DECODE_ABORT_LENGTH;
 
     /* Example of message:
      * 01001001 00011010 00000100
@@ -47,7 +47,8 @@ static int rftech_callback(r_device *decoder, bitbuffer_t *bitbuffer) {
      */
     sensor_id = bb[r][0];
     value = (bb[r][1] & 0x7f) + (bb[r][2] & 0x0f) / 10.0;
-    if(bb[r][1] & 0x80) value = -value;
+    if (bb[r][1] & 0x80)
+        value = -value;
 
     battery = (bb[r][2] & 0x80) == 0x80;
     button = (bb[r][2] & 0x60) != 0;
@@ -56,8 +57,8 @@ static int rftech_callback(r_device *decoder, bitbuffer_t *bitbuffer) {
             "model", "", DATA_STRING, "RF-tech",
             "id", "Id", DATA_INT, sensor_id,
             "battery", "Battery", DATA_STRING, battery ? "OK" : "LOW",
-            "button", "Button", DATA_INT, button,
             "temperature_C", "Temperature", DATA_FORMAT, "%.01f C", DATA_DOUBLE, value,
+            "button", "Button", DATA_INT, button,
             NULL);
 
     decoder_output_data(decoder, data);
@@ -69,9 +70,9 @@ static char *csv_output_fields[] = {
     "model",
     "id",
     "battery",
-    "button",
     "temperature_C",
-    NULL
+    "button",
+    NULL,
 };
 
 r_device rftech = {
